//*******************************************************
//
//	IrCtrl2
//
//	Ir信号処理
//
//	MCU : ATmega88
//		efuse:F5h
//		hfuse:D5h	BOD 2.7V
//		lfuse:E7h	4MHz セラロック
//
//	Compiler:	AVR-GCC		4.8.2
//				AVR-LIBC	1.8.0
//				binutils	2.23.1
//
//	2014.6.15	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"com.h"
#include	"ir.h"

IR_DATA		IrData;
uchar		IrDataFlag;

extern COM_WORK	Com0;

extern volatile uchar	IrTimer;
extern volatile uchar	RecTimer;			// 録画タイミング
extern volatile uchar	RecTimerDS;

volatile uchar	IrIntFlag;			// Ir信号割り込みフラグ
volatile ushort	PulseInt;			// Ir信号間隔
volatile ushort	PulseWidth;			// Ir信号パルス幅

volatile uchar	IrStat;
volatile uchar	IrIntCount;


//*******************************************************
//	アナログコンパレータ割り込み
//	Ir信号の立ち上がりと立ち下がりで発生する
//*******************************************************
ISR(ANALOG_COMP_vect){
	ushort	tm;

	tm = TCNT1L;
	tm |= ((ushort)TCNT1H) << 8;

	if (IS_IR){			// Ir信号 ON
		TCNT1H = 0;
		TCNT1L = 0;
		PulseInt = tm - PulseWidth;
		IrIntFlag = 1;
	} else {			// Ir信号 OFF
		PulseWidth = tm;
		IrIntFlag = 2;
	}
}


//*******************************************************
//	TIMER1 オーバーフロー割り込み
//*******************************************************
ISR(TIMER1_OVF_vect){

	IrStat = 0;		// 長時間変化が無いので Ir信号処理シーケンス初期化
}


//*******************************************************
//	タイマー2比較一致B割り込み
//	38kHz
//*******************************************************
ISR(TIMER2_COMPB_vect){

	IrIntCount++;
}


//*******************************************************
//	Ir 1パルス送信
//*******************************************************
void ir_pulse(uchar width){

	IrIntCount = 0;
	IR_OUT_ON;
	while(IrIntCount < width);
	IR_OUT_OFF;
}


//*******************************************************
//	Ir送信
//*******************************************************
void ir_send(PGM_VOID_P ir_data){
	uchar	format;
	int		leader;
	uchar	start_int;
	uchar	int1;
	uchar	t;
	uchar	c;
	uchar	len;
	int		i, bit;

	WDR;

	format = pgm_read_byte(ir_data++);
	if (format == IR_FORMAT_NEC){
		t = IR_NEC_T;
		leader = IR_NEC_T16;
		start_int = IR_NEC_T8;
		int1 = IR_NEC_T3;
	} else if (format == IR_FORMAT_AEHA){
		t = IR_AEHA_T;
		leader = IR_AEHA_T8;
		start_int = IR_AEHA_T4;
		int1 = IR_AEHA_T3;
	} else {
		return;
	}

	// Leader
	c = IrIntCount = 0;
	IR_OUT_ON;
	for (i = 0; i < leader; i++){
		while(c == IrIntCount);
		c = IrIntCount;
	}
	IR_OUT_OFF;

	IrIntCount = 0;
	while(IrIntCount < start_int);

	len = pgm_read_byte(ir_data++);
	for (i = 0; i < len; i++){
		uchar	l;
		c = pgm_read_byte(ir_data++);
		for (bit = 0; bit < 8; bit++){
			ir_pulse(t);
			if (c & (1 << bit)){
				l = int1;
			} else {
				l = t;
			}
			IrIntCount = 0;
			while(IrIntCount < l);
		}
	}
	ir_pulse(t);
}


//*******************************************************
//	Irリモコン信号受信処理
//	NECフォーマットと AEHAフォーマットに対応
//*******************************************************
void job_ir(void){
	static uchar	bit_mask = 0;
	static uchar	byte_count = 0;
	static uchar	int_th = 0, t_min = 0, t_max = 0;
	static uchar	maxlen = 0;
	static IR_DATA	ir;

	if ((IrStat == 2)&&(!IrTimer)){	// タイムアウト
		IrStat = 3;
	} else if (IrIntFlag != 2){
		return;
	}

	switch(IrStat){
	  case 0:	// リーダー待ち
		if (((PulseWidth >= IR_NEC_RECV_T_MIN * 16)&&
			 (PulseWidth <= IR_NEC_RECV_T_MAX * 16))||
			((PulseWidth >= IR_AEHA_RECV_T_MIN * 8)&&
			 (PulseWidth <= IR_AEHA_RECV_T_MAX * 8))){
			LED_ON;
			byte_count = 0;
			bit_mask = 1;
			memset(&ir, 0, sizeof(IR_DATA));
			IrStat = 1;
#if 0
		} else {
			char	msg[32];
			sprintf_P(msg, PSTR("[0] w:%d\t"), PulseWidth);
			write_str(&Com0, (uchar *)msg);
#endif
		}
		break;

	  case 1:	// スタートビット待ち
		if ((PulseWidth >= IR_NEC_RECV_T_MIN)&&
			(PulseWidth <= IR_NEC_RECV_T_MAX)&&
			(PulseInt >= IR_NEC_RECV_T_MIN * 8)&&
			(PulseInt <= IR_NEC_RECV_T_MAX * 8)){
			ir.format = IR_FORMAT_NEC;		// NECフォーマット
			int_th = PulseInt / 4;
			maxlen = 4;
		} else if ((PulseWidth >= IR_AEHA_RECV_T_MIN)&&
				   (PulseWidth <= IR_AEHA_RECV_T_MAX)&&
				   (PulseInt >= IR_AEHA_RECV_T_MIN * 4)&&
				   (PulseInt <= IR_AEHA_RECV_T_MAX * 4)){
			ir.format = IR_FORMAT_AEHA;		// AEHAフォーマット
			int_th = PulseInt / 2;
			maxlen = IR_DATA_MAXLEN;
#if 0
		} else {
			char	msg[32];
			sprintf_P(msg, PSTR("[1] w:%d i:%d\t"), PulseWidth, PulseInt);
			write_str(&Com0, (uchar *)msg);
#endif
		}

		if (ir.format){
			int	i;
			i = (int)PulseWidth * 8;
			i /= 10;
			t_min = (uchar)i;
			i = (int)PulseWidth * 12;
			i /= 10;
			t_max = (uchar)i;
			IrTimer = 2;
			IrStat = 2;
		} else {
#if 0
			char	msg[32];
			sprintf_P(msg, PSTR("[1] w:%d i:%d\r\n"), PulseWidth, PulseInt);
			write_str(&Com0, (uchar *)msg);
			//cputs(PSTR("*"));
#endif
			LED_OFF;
			IrStat = 0;
		}
		break;

	  case 2:	// データサンプリング
		if (IrIntFlag == 2){
			IrTimer = 2;
			if ((PulseWidth >= t_min)&&
				(PulseWidth <= t_max)){
				if (PulseInt > int_th){
					ir.data[byte_count] |= bit_mask;
				}
				bit_mask <<= 1;
				if (bit_mask == 0){
					bit_mask = 1;
					++byte_count;
					ir.len = byte_count;
					if (byte_count >= maxlen){
						IrStat = 3;	// 受信完了
						break;
					}
				}
#if 0
			} else {
				char	msg[32];
				sprintf_P(msg, PSTR("[2] w:%d i:%d\r\n"), PulseWidth, PulseInt);
				write_str(&Com0, (uchar *)msg);
				//cputs(PSTR("*"));
				IrStat = 0;
				LED_OFF;
#endif
			}
		}

		break;

	  case 3:	// データ取得完了
		break;

	  default:
		IrStat = 0;
		LED_OFF;
		break;
	}

	// データ取得完了
	if (IrStat == 3){
		if (ir.len > 0){
			int		i;
			char	buff[4];

			IrData = ir;		// グローバル変数へコピー
			IrDataFlag = 1;

			if (ir.format == IR_FORMAT_NEC){
				cputs(PSTR("[NEC:"));
			} else if (ir.format == IR_FORMAT_AEHA){
				cputs(PSTR("[AEHA:"));
			}
			for(i = 0; i < ir.len; i++){
				if (i)	cputs(PSTR(","));
				hex2(ir.data[i], buff);
				write_str(&Com0, (uchar *)buff);
			}
			cputs(PSTR("]\r\n"));

		}
		IrStat = 0;
		LED_OFF;
	}

	IrIntFlag = 0;
}


//*******************************************************
//	IR_DATAを比較
//*******************************************************
int ir_equal(IR_DATA *data, PGM_VOID_P code){
	int	i;
	uchar	d;

	if (data == NULL)		return 0;	//無効
	if (data->format == 0)	return 0;	// 無効
	d = pgm_read_byte(code++);
	if (data->format != d)	return 0;
	d = pgm_read_byte(code++);
	if (data->len != d)		return 0;

	for (i = 0; i < data->len; i++){
		d = pgm_read_byte(code++);
		if (data->data[i] != d)	return 0;
	}
	return 1;		// 同じ
}


//*******************************************************
//	ucharを16進数2桁文字列に変換
//*******************************************************
void hex2(uchar ch, char buff[]){
	int	i;

	buff[2] = 0;
	for (i = 1; i >= 0; --i){
		uchar	c;
		c = (ch & 0x0F) + '0';
		if (c > '9'){
			c += 'A' - '9' - 1;
		}
		buff[i] = c;
		ch >>= 4;
	}
}

