//*******************************************************
//
//	IrCtrl2
//
//	処理タスク
//
//	MCU : ATmega88
//		efuse:F5h
//		hfuse:D5h	BOD 2.7V
//		lfuse:E7h	4MHz セラロック
//
//	Compiler:	AVR-GCC		4.8.2
//				AVR-LIBC	1.8.0
//				binutils	2.23.1
//
//	2014.9.9	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"com.h"
#include	"ir.h"
#include	"task.h"

#define		USE_PRINT_SEQ	1	// シーケンス番号を表示するか
#define		USE_IR			1	// 電源ボタンと録画ボタンを使わずに赤外線操作する

extern COM_WORK	Com0;
extern IR_DATA	IrData;
extern uchar	IrDataFlag;

extern volatile uchar	PowerOffTimer;		// 電源ボタンのON時間
extern volatile uchar	RecOffTimer;		// 録画ボタンのON時間
extern volatile uchar	PowerLedTimer;		// 電源LEDの点灯判断
extern volatile uchar	PowerLedTimerDS;
extern volatile uchar	PowerLedOnTimer;	// 電源LEDの点滅判断
extern volatile uchar	RecLedTimer;		// 録画LEDの点灯判断
extern volatile uchar	RecLedTimerDS;
extern volatile uchar	RecTimer;			// 録画タイミング
extern volatile uchar	RecTimerDS;

extern volatile uchar StatTimer;			// シーケンス処理タイマー
extern volatile uchar StatTimerDS;

uchar	Stat;
uchar	Command;


// VTRコマンド(NEC Video1)
const PROGMEM IR_DATA VTRCodePowerOn = {
	IR_FORMAT_NEC,
	4,
	{ 0x19, 0xE2, 0x5B, 0xA4 }
};
const PROGMEM IR_DATA VTRCodeRec = {
	IR_FORMAT_NEC,
	4,
	{ 0x19, 0xE2, 0x15, 0xEA }
};
const PROGMEM IR_DATA VTRCodeStop = {
	IR_FORMAT_NEC,
	4,
	{ 0x19, 0xE2, 0x10, 0xEF }
};


// レコーダへ送信するコマンド(NEC)
const PROGMEM IR_DATA RecoderCodePowerOn = {
	IR_FORMAT_NEC,
	4,
	{ 0xFD, 0x03, 0x14, 0xEB }
};
const PROGMEM IR_DATA RecoderCodeRec = {
	IR_FORMAT_NEC,
	4,
	{ 0xFD, 0x03, 0x03, 0xFC }
};
const PROGMEM IR_DATA RecoderCodeOK = {
	IR_FORMAT_NEC,
	4,
	{ 0xFD, 0x03, 0x07, 0xF8 }
};
const PROGMEM IR_DATA RecoderCodeReturn = {
	IR_FORMAT_NEC,
	4,
	{ 0xFD, 0x03, 0x0D, 0xF2 }
};


// CATVチューナーへ送信するコマンド(AEHA)
const PROGMEM IR_DATA TunerCodePowerOn = {
	IR_FORMAT_AEHA,
	6,
	{ 0x02, 0x20, 0x80, 0x26, 0x8D, 0x2B }
};


//*******************************************************
//	処理タスク
//*******************************************************
void job_task(void){
	static uchar	last_stat = 0;
	static uchar	retry_count = 0;
	static uchar	stop_rec_retry = 0;

	// Ir信号が来たか
	if (IrDataFlag){
		// ↓Ir送信側で想定している電源状態
		static uchar	power_on_flag = 0;
		if (ir_equal(&IrData, &VTRCodePowerOn)){
			if (power_on_flag){		// 電源が入っているなら
				power_on_flag = 0;
				Command = COMMAND_POWEROFF;
			} else {
				Command = COMMAND_START;
			}
		} else if (ir_equal(&IrData, &VTRCodeRec)){
			Command = COMMAND_REC;
			RecTimerDS = 0;
			RecTimer = REC_WAIT;
			power_on_flag = 1;	// 電源が入っていることを想定して送信している
		} else if (ir_equal(&IrData, &VTRCodeStop)){
			Command = COMMAND_STOP;
			power_on_flag = 1;	// 電源が入っていることを想定して送信している
		}
		IrDataFlag = 0;
	}


	// 処理シーケンス
	switch(Stat){

	  case STAT_POWEROFF:		// 電源OFF
		if (PowerLedTimer){		// 外部から電源ONされた
			Stat = STAT_READY;
			Command = 0;
		} else if (Command == COMMAND_START){	// 電源ONコマンド
			StatTimerDS = 0;
			StatTimer = POWER_ON_WAIT;
			Stat = STAT_POWERONWAIT;
		} else if (Command == COMMAND_REC){		// 録画コマンドで起動
			StatTimerDS = 0;
			StatTimer = 0;
			Stat = STAT_POWERONWAIT;
		} else if (Command){
			Command = 0;
		}
		break;

	  case STAT_POWERONWAIT:	// 電源を入れるまで時間待ち
		if (StatTimer == 0){
			power_on();			// 電源ON
			StatTimer = 5;
			Stat = STAT_WAITPOWERON;
		}
		break;

	  case STAT_WAITPOWERON:
		if (StatTimer == 0){
			if (PowerLedTimer){		// 電源が入ったか
				Stat = STAT_WAITRECREADY;
				StatTimer = READY_WAIT;
			} else {
				Stat = STAT_POWERONWAIT;	// 再度投入
			}
		}
		break;

	  case STAT_WAITRECREADY:	// 録画可能になるまで待つ
		if (PowerLedOnTimer == 0xFF){
			Stat = STAT_STARTING;
		} else if (StatTimer == 0){
			// 電源投入エラー
			send_ok();		// [OK]
			Stat = STAT_STARTING;
		}
		break;
		
	  case STAT_STARTING:		// 録画時間まで待つ
		if (RecTimer == 0){
			Stat = STAT_READY;
		}
		break;

	  case STAT_READY:		// 待機状態
		if (RecLedTimer){				// 外部で録画スタートした
			Stat = STAT_REC;
		} else if (!PowerLedTimer){		// 外部から電源OFF
			Command = 0;
			Stat = STAT_POWEROFF;
		} else if (Command == COMMAND_REC){	// 録画コマンド
			start_rec();
			StatTimerDS = 0;
			StatTimer = REC_START_TIME;
			Stat = STAT_STARTREC;
		} else if (Command == COMMAND_POWEROFF){	// 電源OFFコマンド
			Command = 0;
			power_off();
			StatTimerDS = 0;
			StatTimer = POWER_OFF_TIME;		// シャットダウン時間
			Stat = STAT_SHUTDOWN;
		} else if (Command == COMMAND_START){	// 視聴中に録画予約発生
			Command = 0;
			send_return();		// [戻る]
			retry_count = 0;
			StatTimerDS = 0;
			StatTimer = 5;
			Stat = STAT_RETURN;
		} else if (Command){
			Command = 0;
		}
		break;

	  case STAT_RETURN:			// 録画可能状態に戻す
		if (StatTimer == 0){
			send_return();		// [戻る]
			if (++retry_count < 3){
				StatTimer = 5;
			} else {
				Stat = STAT_READY;
			}
		}
		break;

	  case STAT_STARTREC:		// 録画開始
		if (StatTimer == 0){
			if (!RecLedTimer){		// 録画状態にならなかった
				Stat = STAT_READY;	// 再度
			} else {
				Command = 0;
				Stat = STAT_REC;
			}
		}
		break;

	  case STAT_REC:			// 録画中
		if (!RecLedTimer){		// 外部要因で停止した
			Stat = STAT_READY;
		} else if ((Command == COMMAND_STOP)||		// STOPコマンド
				   (Command == COMMAND_POWEROFF)){	// 電源OFFコマンド
			//stop_rec();	ここではまだ止めない
			StatTimer = 5;	// 5秒以内に CATVチューナー ON
			StatTimerDS = 0;
			Stat = STAT_TUNER_ON;
		}
		break;

	  case STAT_TUNER_ON:
		if ((StatTimer == 0)||
			(Command == COMMAND_POWEROFF)){
			tuner_on_off();			// CATVチューナー ON
			stop_rec_retry = 0;		// 停止リトライカウント
			StatTimer = 1;			// Ir信号が連続するのを避ける為、1秒後
			Stat = STAT_STOPREC;
		}
		break;

	  case STAT_STOPREC:
		if (StatTimer == 0){
			if (!PowerLedTimer){		// 電源が切られた
				Command = 0;
				Stat = STAT_POWEROFF;
			} else if (!RecLedTimer){	// 録画停止した
				StatTimer = SHUTDOWN_WAIT;
				Stat = STAT_SAVE;
			} else if (stop_rec_retry < 3){	// 最大３回停止操作をする
				stop_rec_retry++;
				stop_rec();			// 録画停止
				StatTimer = 7;		// 7秒後に再度状態確認
			} else {
				// 録画停止失敗したので強制電源OFF
				force_power_off();
				StatTimer = POWER_OFF_TIME;
				Stat = STAT_SHUTDOWN;
			}
		}
		break;
		
	  case STAT_SAVE:		// SAVE終了処理・次の録画命令待ち
		if (Command == COMMAND_REC){	// 次の録画
			Stat = STAT_READY;
		} else if (StatTimer == 0){
			power_off();	// 電源OFFコマンドが来なくても電源OFFする
			StatTimer = POWER_OFF_TIME;
			Stat = STAT_SHUTDOWN;
		}
		break;

	  case STAT_SHUTDOWN:	// 停止中
		if (!PowerLedTimer){
			POWER_SW_OFF;
			REC_SW_OFF;
			tuner_on_off();			// CATVチューナー OFF
			Stat = STAT_POWEROFF;
		} else if (StatTimer == 0){
			power_off();	// 再度OFF
			StatTimer = POWER_OFF_TIME;
		}
		break;

	  case STAT_ERROR:	// エラー
		cputs(PSTR("{ERROR}"));
		POWER_SW_OFF;
		REC_SW_OFF;
		Stat = STAT_POWEROFF;
		break;
	}

#if USE_PRINT_SEQ
	// シーケンス動作確認
	if (Stat != last_stat){
		char	msg[16];
		cputs(PSTR("("));
		itoa(Stat, msg, 10);
		write_str(&Com0, (uchar *)msg);
		cputs(PSTR(")"));
		last_stat = Stat;
	}
#endif
}


//*******************************************************
//	リモコン「戻る」
//*******************************************************
void send_return(void){

	cputs(PSTR("<RETURN>\r\n"));
	ir_send(&RecoderCodeReturn);	// [戻る]
}


//*******************************************************
//	リモコン「OK」
//*******************************************************
void send_ok(void){

	cputs(PSTR("<OK>\r\n"));
	ir_send(&RecoderCodeOK);	// [OK]
}


//*******************************************************
//	CATVチューナー ON/OFF
//*******************************************************
void tuner_on_off(void){

	cputs(PSTR("<Tuner ON/OFF>\r\n"));
	ir_send(&TunerCodePowerOn);		// CATVチューナー ON/OFF
}


//*******************************************************
//	電源ON
//*******************************************************
void power_on(void){

	cputs(PSTR("<ON>\r\n"));

#if USE_IR
	ir_send(&RecoderCodePowerOn);	// [ON/OFF]
#else
	POWER_SW_ON;
	PowerOffTimer = SWITCH_TIME;
#endif
}


//*******************************************************
//	録画開始
//*******************************************************
void start_rec(void){

	cputs(PSTR("<REC>\r\n"));

#if USE_IR
	ir_send(&RecoderCodeRec);	// [Rec]
#else
	REC_SW_ON;
	RecOffTimer = SWITCH_TIME;
#endif
}


//*******************************************************
//	録画停止
//*******************************************************
void stop_rec(void){

	cputs(PSTR("<STOP>\r\n"));

#if USE_IR
	ir_send(&RecoderCodeRec);	// [Rec]
#else
	REC_SW_ON;
	RecOffTimer = SWITCH_TIME;
#endif
}


//*******************************************************
//	電源OFF
//*******************************************************
void power_off(void){

	cputs(PSTR("<OFF>\r\n"));

#if USE_IR
	ir_send(&RecoderCodePowerOn);	// [ON/OFF]
#else
	POWER_SW_ON;
	PowerOffTimer = SWITCH_TIME;
#endif
}


//*******************************************************
//	強制電源OFF
//*******************************************************
void force_power_off(void){

	cputs(PSTR("<Force OFF>\r\n"));

	POWER_SW_ON;
	PowerOffTimer = 0;	// 電源が切れるまでON継続
}


